;*******************************************************************************;
;   Microchip Technology Inc   2006
;	
;   ASP 201    LAB 1
;   Basic Interrupt Enabling
;   v2.3 revised 10/3/2006 by Stu Chandler, Sr.TTE
;   Modify for APP001 EVM board 4/13'2007 by Richard Yang, Pr.CAE
;
;   This lab gives the user the basic startup routine, a control loop and a 
;   bare-bones interrupt vector.  When completed, downloaded and running to the 
;   APP001 will increment 8-LEDs every time when the "SW2" is pressed
;
;   Users must add 3 lines of code in order to get this program to run.
;   1. The first line of code enables the INT0 interrupt to occur when RB0 (SW2)
;      is pressed. 
;	2. The second line of code to be added by the user enables global interrupts
;	3. The last line of code clear the INT0 interrupt flag in ISR routine.
;
;   To verify the code is operating, enable a break point in the 
;   interrupt vector.  When the break point is set every time SW2 is 
;   pushed the program will break in the interrupt vector and allow the user to 
;   see the LEDs increment on the PORTD. 
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;      
;*******************************************************************************;
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location

Db_Ctr1     RES 1                   ; pair of counters used to create delay 
Db_Ctr2     RES 1                   ;  for debounce routine


;=================================== PROGRAM MEMORY

;-----------------------------------
; VECTORS
;-----------------------------------
			
RESET_V     CODE    0x0000          		; reset vector
    		goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          		; interrupt vector
    		goto    Int_Service_Routine     ; jump to interrupt service routine
    
        
;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;
	banksel	ANSELH					; switch to bank 3
	bcf		ANSELH,ANS12			; disable AD input function for RB0/AN12
;
	banksel TRISB             		; switch to bank 1   
    bsf     TRISB,0    				; Set SW2 (RB0) as input pin
	clrf	TRISD					; Set LED (PORTD) for output function
;
	banksel PORTD					; switch back to bank 0	
    movlw	0x01
	movwf   PORTD              		; Turn LED0 on the PORTD
;
    clrf    Db_Ctr1
    clrf    Db_Ctr2
     
	bcf     INTCON,INTF                
    bsf     INTCON,INTE             ; ### Enable INT0 Interrupt
    bsf     INTCON,GIE              ; ### Enable Global Interrupt									 		
;
Loop
    nop
    goto    Loop 


;-----------------------------------
; Subroutine:   Debounce
; Parameters:   None
; Returns:      Nothing
; Synopsys:     Debounce routine for S3
;-----------------------------------

Debounce							; allows time for SW2 to debounce
    incfsz  Db_Ctr1,F
    goto    $-1
    incfsz  Db_Ctr2,F
    goto    Debounce
    return 	

		
;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Nothing
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
    call    Debounce                ; Delay until switch stops bouncing  
    incf    PORTD,F            		; increments number of time button has been pushed
    bcf     INTCON,INTF             ; ### clears the INT0 Interrupt Flag
    retfie     
		
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;
