;****************************************************************
;   This file is a example code for the Internal EEDATA access  *
;   on the PICmicro PIC16F887                                 	*  
;                                                               *
;   How aboud userful in the example code:						*
;   1. Know the used the directive instruction to define 		*
;      the EEDATA with MAPSM code.								*
;	2. Read the Internal EEDATA using assembly					*
;	3. Write data into the Inerrnal EEDATA using assembly		*
;                                                               *
;****************************************************************
;                                                               *
;    Filename:Lab10-EEPROM.asm        	                *
;    Date: 21st March 2005                                      *
;    File Version: v1.0                                         *
;                                                               *
;    Author: Richard Yang                                       *
;    Company:  Microchip Technology                             *
;                                                  	        * 
;                                                               *
;****************************************************************

;=================================== DATA MEMORY
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.
;
;***************     Variable Definitions in Share Bank   ********************
; Reserve RAM location 0x70 for ICD

w_temp			EQU	0x71		; variable used for context saving 
status_temp		EQU	0x72		; variable used for context saving
pclath_temp		EQU	0x73		; variable used for context saving
;
Data_EE_Addr   	equ   	0X74
Data_EE_Data   	equ   	0X75

;*************     Declare EEPROM Data Byte   ********************
;
; Syntax:     [<label>] de <expr> [, <expr>, ..., <expr>]
;
; Reserve memory words with 8-bit data. Each <expr> must evaluate
; to an 8-bit value. The upper bits of the program word are zeroes. 
; Each character in a string is stored in a separate word.
; Although designed for initializing EEPROM data on the PIC16F877A,
;
; As following example code will generated the EEDATA into the Hex file
;
;		org	0x2100		; Set EEDATA location at 0x00
;
;		DE	"Microchip Technology", 0x00
;		DE	"Firmware Version V2.30", 0x00
;
;		org	0x2180		; Set EEDATA location at 0x80
;
;		DE	'a','b','c','d', 0x0A, 0x0D, 0x00,0x00

;----------------------------------------------------------------------
; If you are using relocatable syntax, please use the CODE section 
; The section declare have been defined in the 16f877a.lkr
;
;              Declare EEDATA in the 16f877a.lkr
; CODEPAGE   NAME=eedata   START=0x2100   END=0x21FF   PROTECTED
; SECTION    NAME=DEEPROM  ROM=eedata     	 
;----------------------------------------------------------------------
; Example code for using the linker
;
DEEPROM		code 			; Put following data into Internal EEPROM
			DE	"Microchip Technology", 0x00
			DE	"Firmware Version V2.30", 0x00

;***************        Program Vectors     **********************
;
Reset_V     code   	0x000 
			nop
         	goto  	Main

ISR_V		code    0x004             ; interrupt vector location
			goto	ISR_Service
;
			code	0x020
ISR_Service
		movwf   w_temp            	; save off current W register contents
		movf	STATUS,w          	; move status register into W register
		movwf	status_temp       	; save off contents of STATUS register
		movf	PCLATH,w	  		; move pclath register into w register
		movwf	pclath_temp	  		; save off contents of PCLATH register

; isr code can go here or be located as a call subroutine elsewhere
;		call	isr_routine

		movf	pclath_temp,w	  	; retrieve copy of PCLATH register
		movwf	PCLATH		  		; restore pre-isr PCLATH register contents
		movf    status_temp,w     	; retrieve copy of STATUS register
		movwf	STATUS            	; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          	; restore pre-isr W register contents
		retfie                    	; return from interrupt

;***************        Program Start      **********************
;
Main
        call   	Init_EEdata	; Initialization
;
;***************************************************************
;  Following code will write the SSCII code from 'A' to 'P'
;  character code into the EEPROM location from 0x30 to 0x3F
;***************************************************************
;
		movlw	0x30		; Load the EEDATA point address to location at 0x30
		movwf	Data_EE_Addr
		movlw	'A'		; Load the start ASCII character "A" 
		movwf	Data_EE_Data
EE_Write
		call	WRITE_EEPROM	; Write a byte into EEPROM
		incf	Data_EE_Data,F	; 'A' to 'P'
		incf	Data_EE_Addr,F  ; Point to next EEPROM Address
		btfss	Data_EE_Addr,6	; have writed 16 bytes ? (end of addr = 0x40, b'01000000'
		goto	EE_Write
		nop			
; You can put a BREAK point here to view the EEPROM window under MPLAB IDE
;
;***************************************************************
;  Following code will read the contents from EEPROM and the
;  data will put with RAM location from 0x20 to 0x5F
;
; Addr                 Contents                             ASCII Code
;------  -----------------------------------------------  ------------------
; 0020:  4D 69 63 72 6F 63 68 69 70 20 54 65 63 68 6E 6F  Microchi p Techno
; 0030:  6C 6F 67 79 00 46 69 72 6D 77 61 72 65 20 56 65  logy.Fir mware Ve
; 0040:  72 73 69 6F 6E 20 56 32 2E 33 30 00 FF FF FF FF  rsion V2 .30.....
; 0050:  41 42 43 44 45 46 47 48 49 4A 4B 4C 4D 4E 4F 50  ABCDEFGH IJKLMNOP
;
;***************************************************************
;
		movlw	0x20		; Set RAM Pointer start address at 0x20
		movwf	FSR
;		
		movlw	0x0		; Set pointer to EEPROM start address at 0x00
		movwf	Data_EE_Addr
EE_Read
		call	READ_EEPROM	; Read EEPROM data with curent address to W registre
		movwf	INDF		; move W --> RAM 
;
		incf	FSR,F		; RAM Pointer + 1
		incf	Data_EE_Addr,F  ; EEPROM address pointer + 1
		btfss	Data_EE_Addr,6	; Read 64 bytes ? (Addr = 0x40 ,b'01000000')
		goto	EE_Read
;    
        GOTO   	$   		; Loop here
;
;------ INTERNAL EEPROM READ ------
;
READ_EEPROM
		banksel	EEADR
		movf	Data_EE_Addr,W		; Load EEPROM Address into Pointer
		movwf	EEADR
;
		banksel	EECON1
		bcf	EECON1,EEPGD		; Read from EEPROM ; * For 16F877 (select Flash or EEPROM)
		bsf	EECON1,RD
;
		banksel	EEDATA
		movf	EEDATA,W
;
		banksel	PORTA
		return
;	
;----INTERNAL EEPROM WRITE-----
;
WRITE_EEPROM
		banksel	EEADR		
		movf	Data_EE_Addr,W		; Load EEPROM Address & Data into Register
		movwf	EEADR
		movf	Data_EE_Data,W
		movwf	EEDATA
;		
		banksel	EECON1
		bcf	EECON1,EEPGD		; Selected Data EEPROM for Writing
		bsf	EECON1,WREN		; Enable write function
;
;		bcf	INTCON,GIE		; Disable Interrupt, if needed
;
		movlw	0x55			; Write Sequence control 
		movwf	EECON2	
		movlw	0xAA
		movwf	EECON2	
;			
		bsf	EECON1,WR		; Start EEPROM write operation
;
;		bsf	INTCON,GIE		; Enable Interrupt (If used interrupt )
		bcf	EECON1,WREN		; Disable write EEPROM  
;
		btfsc	EECON1,WR	; Waiting for writing completed (4mS)
		goto	$-1			; During write period, must hold both EEADR & EEDATA until complete
;
		banksel	PORTA
		return		
;
;---------INITIAL----------------
;
Init_EEdata      
        BCF     PIR2,EEIF
		banksel	PIE2			; Bank swap for PIE2
        BCF     PIE2,EEIE		; Disale EEDATA interrupt
		banksel PORTA			; select bank0 for return
        RETURN
;--------------------------------
        END   
