;*******************************************************************************;
;   Microchip Technology Inc   2006
;
;   ASP 201     LAB 2
;   Timer 1 lab
;   v2.0 revised 9/29/2006 by Stu Chandler, Sr.TTE
;		
;   This lab uses Timer 1 to interrupt the PIC 10 times a second
;
;   The interrupt vector given toggles the LED connected to PORTD<RD0> on every
;   interrupt and then toggles the LED on PORTD<RD2> every fifth interrupt.  
;
;   To achieve this interrupt rate TMR1 must be set up with the following:
;
;       clock source  = Fosc/4 = 1 Mhz
;       Pre-scaler	  =  2
;       Pre-load value = 15,536  ( -50,0000) ( 0x3CB0) 
;
;   Two sections of code must be completed to finish this lab:
;
;   The first section of code initially sets TMR1 to 15,536, selects
;   the clock source, and turns on TMR1
;
;   The second section of code enables Timer 1 interrupts, enables Peripheral 
;   interrupts and finally enables Global interrupts
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;		      
;*******************************************************************************;
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions	

;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location

Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register
Count		RES 1                   ; variable used to Count interrupts

#define		LEDs	PORTD
#define		LED0	PORTD,0
#Define		LED2	PORTD,2

;=================================== PROGRAM MEMORY			
			
;-----------------------------------
; VECTORS
;-----------------------------------

RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          ; interrupt vector
    goto    Int_Service_Routine     ; jump to interrupt service routine
    

;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;	
	banksel LEDs
    clrf    LEDs                    ; Make sure all LEDs turn off
    movlw   0x05                    ; initialize Count variable
    movwf   Count

    BANKSEL TRISD                   ; switch to bank containing TRISD
    movlw   0x00          		   	; config PORTD as outputs
    movwf   TRISD
    BANKSEL 0                       ; switch to bank 0                     

;-----------------------------------

; select clock source, init pre-scaler, load 0x3CB0 into TMR1, and turn it on
		
; ### initialize TMR1L and TMR1H
    movwf   TMR1H                    
; ###    
    movwf   TMR1L                    
        
; ### set pre-scaler to 2
; ### set clock source to Fosc/4
; ### turn TMR1 on				

; enable TMR1 interrupts, Peripheral Interrupts, and Global Interrupts

    bsf     STATUS,RP0              ; switch to bank 1
; ### enable TMR1 Interrupts
; ### enable Peripheral Interrupts
; ### enable Global interrups
    bcf     STATUS,RP0              ; return to bank 0	
	
;-----------------------------------

Loop
    nop
    goto    Loop

         
;-----------------------------------
; Subroutine:   Save_Regs
; Parameters:   WREG, STATUS, PCLATH
; Returns:      Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Synopsys:     Saves WREG, STATUS & PCLATH upon entering interrupt
;-----------------------------------

Save_Regs                           ; routine to save W, STATUS, & PCLATH
    movwf   Tmp_WREG                ; save WREG in temp register			
    swapf	STATUS,W                ; swapf does not clobber status bits
    movwf	Tmp_STATUS              ; save STATUS in temp register
    movf	PCLATH,W                ; save PCLATH
    movwf	Tmp_PCLATH
    return


;-----------------------------------
; Subroutine:   Restore_Regs
; Parameters:   Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Returns:      WREG, STATUS, PCLATH
; Synopsys:     Restores WREG, STATUS & PCLATH after interrupt
;-----------------------------------		

Restore_Regs						; rountine to restore W, STATUS, & PCLATH
    movf    PCLATH,W                ; restore PCLATH
    movwf   PCLATH
    swapf   Tmp_STATUS,W            ; restore STATUS
    movwf   STATUS
    swapf   Tmp_WREG,F              ; restore W without clobbering STATUS reg
    swapf   Tmp_WREG,W
    return
		
						
;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Multiple
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
    call    Save_Regs               ; save W, STATUS, & PCLATH
    BANKSEL	0				        ; select bank 0
    bcf	    PIR1,TMR1IF             ; clear TMR1 int request flag

; Load Timer 1 with value that will overflow in 50,000 cycles

    movlw   (.65536-.50000)/.256    ; initialize TMR1L and TMR1H
    movwf   TMR1H                   
    movlw   (.65536-.50000)%.256    
    movwf   TMR1L                  

; toggle LED 0 on every interrupt
; toggle LED 2 on every fifth interrupt

    movlw   0x01
    xorwf   LEDs,F                 ; toggle LED0 on PORTB<RD0> 
    decfsz  Count,F                 ; was this the fifth interrupt?
    goto    Finish_Int              ;   no: finish interrupt routing
    movlw   0x05                    ;   yes:toggle LED2 on PORTB<RB2>
    movwf   Count
    movlw   0x04
    xorwf   LEDs,F                 ; toggle PORTD<RD3>
    				
Finish_Int                          ; restore W,STATUS & PCLATH
    call    Restore_Regs
    retfie     
		
		
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;
