;*******************************************************************************;
;   Microchip Technology Inc   2006
;
;   ASP 201     LAB 3
;   Timer 2 lab
;   v2.1 revised 9/29/2006 by Stu Chandler, Sr.TTE		
;		
;      
;   This lab provides experience setting up and using TMR2.  The interrupt vector
;   and main program are provided.  When completed this lab will increment a digital
;   count of the interrupts and display the count on the APP001 LEDs.  

;   This labs "slows down" the interrupt rate on TMR2 by a combination of pre-scaler,
;   post-scaler and PR2 value.  When finished the interrupt vector is entered 10mS. 
;   The the LEDs will increased the binary count.
;
;   To achieve this two sections of code must be completed by the student:
;
;   1) The first section sets clock source, pre-scaler, post-scaler, PR2 value, 
;      and turns on Timer 2
;
;   2) The second section enables Timer 2 interrupts, Global interrupts and
;      peripheral interrupts
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;
;*******************************************************************************;
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions	
;=================================== DATA MEMORY	

;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location

Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register
Count		res 1				    ; variable used to count interrupts

#define		LEDs	PORTD

;=================================== PROGRAM MEMORY
			
;-----------------------------------
; VECTORS
;-----------------------------------
			
RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          ; interrupt vector
    goto    Int_Service_Routine     ; jump to interrupt service routine
    
;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;	
	banksel	PORTD
    clrf    LEDs                    ; initialize PORTD
    
    BANKSEL TRISD                   ; switch to bank containing TRISB
    movlw   0x00                    ; config PORTD as LED outputs
    movwf   TRISD
    
    BANKSEL PR2                     ; point to PR2 banks      
    movlw   .250                    ; decimal 250, could also write d'250'
    movwf   PR2                     ; load PR2 with 250 


;-----------------------------------

; configure TMR2 with prescaler = 4, and postscaler = 13

    BANKSEL 0                       ; point to bank 0
    movlw   b'01001000'             ; ### set TMR2 postscaler = 10
    movwf   T2CON                   ; ###		
    bsf     T2CON,T2CKPS0           ; ### set TMR2 prescaler = 4	
    bsf     T2CON,TMR2ON            ; ### turn on TMR2			

; enable TMR2, peripheral, and global interrupts

    bsf     STATUS,RP0              ; point to bank 1
    bsf     PIE1,TMR2IE             ; ### enable TMR2 interrupts	
    bsf     INTCON,PEIE             ; ### enable Peripheral interrupts
    bsf     INTCON,GIE              ; ### enable Global interrups
    bcf     STATUS,RP0				; return to bank 0
    	
	
;-----------------------------------

Loop
    nop
    goto    Loop
 
        
;-----------------------------------
; Subroutine:   Save_Regs
; Parameters:   WREG, STATUS, PCLATH
; Returns:      Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Synopsys:     Saves WREG, STATUS & PCLATH upon entering interrupt
;-----------------------------------

Save_Regs                           ; routine to save W, STATUS, & PCLATH
    movwf   Tmp_WREG                ; save WREG in temp register			
    swapf	STATUS,W                ; swapf does not clobber status bits
    movwf	Tmp_STATUS              ; save STATUS in temp register
    movf	PCLATH,W                ; save PCLATH
    movwf	Tmp_PCLATH
    return
		

;-----------------------------------
; Subroutine:   Restore_Regs
; Parameters:   Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Returns:      WREG, STATUS, PCLATH
; Synopsys:     Restores WREG, STATUS & PCLATH after interrupt
;-----------------------------------

Restore_Regs						; rountine to restore W, STATUS, & PCLATH
    movf    PCLATH,W                ; restore PCLATH
    movwf   PCLATH
    swapf   Tmp_STATUS,W            ; restore STATUS
    movwf   STATUS
    swapf   Tmp_WREG,F              ; restore W without clobbering STATUS reg
    swapf   Tmp_WREG,W
    return
		

;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Multiple
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
    call    Save_Regs               ; save W, STATUS, & PCLATH
    
    BANKSEL	0				        ; select bank 0
    bcf	    PIR1,TMR2IF             ; clear TMR2 int request flag
    incf    LEDs,F                  ; increment count of interupts

Finish_Int                          ; restore W,STATUS & PCLATH
    call    Restore_Regs
    retfie     
		
		
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;

