;*******************************************************************************;
;   Microchip Technology Inc   2006
;
;   ASP 201     LAB 5
;   CCP lab
;   v2.0 revised 10/5/2006 by Stu Chandler, Sr.TTE
;	
;   This lab configures the CCP into output compare mode driven by Timer 1.
;   When the CCP interrupts the vector will toggle the output pin that drives
;   the buzzer.  During the Interrupt processing the value of CCPR1L is decremented.
;   This will continually alter the period of the interrupt.  To finish altering the
;   tone the CCP will set the Special Event flag to reset TMR1L and TMR1H.  The 
;   combination of a reduced period and a reseting of Timer 1 will cause the 
;   buzzer to emit a chirping sound.		
;
;   To complete this lab 3 portions of code must be completed:
;			
;   1) Configrure CCP1 as output compare that sets CCPIF and the Special Event Flag		
;   2) Configure Timer 1 with Fosc/4 as the clock soure and a pre-scaler of 1:8
;   3) Enable CCP, Global and Peripheral interrupts
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;      
;*******************************************************************************;

;=================================== DATA MEMORY
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location
		
Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register


;=================================== PROGRAM MEMORY
			
;-----------------------------------
; VECTORS
;-----------------------------------

RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          ; interrupt vector
    goto    Int_Service_Routine     ; jump to interrupt service routine
    

;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------            

            CODE	0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;	
;-----------------------------------
;   Set CCP1COM to Output Compare mode with Special Event Trigger
;   to clear the Timer 1 register pair on a match

	banksel	CCP1CON
    movlw   0x0B                    ; ### value for CCP1CON
    movwf   CCP1CON                 ; move to CCP1CON

;-----------------------------------
;   Configure Timer 1 for Fosc/4 operation.  8:1 Pre-scaler

    movlw   0x30                    ; ### value for TMR1
    movwf   T1CON                   ; ### move to TMR1 control register
    
    clrf    CCPR1L                  ; init CCP1 to 0x0100
    clrf    CCPR1H
    incf    CCPR1H,F
    			 
    bsf     STATUS,RP0              ; point to BANK1
    bcf     TRISC,2                 ; config CCP2 PORTC<RC2> as output			

;-----------------------------------
;   enable interrupts

    bsf     PIE1,CCP1IE             ; ### enable CCP1 interrupt
    bsf     INTCON,PEIE             ; ### enable peripheral interrupts
    bsf     INTCON,GIE              ; ### enable global interrupts

    bcf     STATUS,RP0              ; point to BANK0
    bsf     T1CON,TMR1ON            ; turn on TMR1
	

;-----------------------------------

Loop
    nop
    goto    Loop

         
;-----------------------------------
; Subroutine:   Save_Regs
; Parameters:   WREG, STATUS, PCLATH
; Returns:      Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Synopsys:     Saves WREG, STATUS & PCLATH upon entering interrupt
;-----------------------------------

Save_Regs                           ; routine to save W, STATUS, & PCLATH
    movwf   Tmp_WREG                ; save WREG in temp register			
    swapf	STATUS,W                ; swapf does not clobber status bits
    movwf	Tmp_STATUS              ; save STATUS in temp register
    movf	PCLATH,W                ; save PCLATH
    movwf	Tmp_PCLATH
    return


;-----------------------------------
; Subroutine:   Restore_Regs
; Parameters:   Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Returns:      WREG, STATUS, PCLATH
; Synopsys:     Restores WREG, STATUS & PCLATH after interrupt
;-----------------------------------		

Restore_Regs						; rountine to restore W, STATUS, & PCLATH
    movf    PCLATH,W                ; restore PCLATH
    movwf   PCLATH
    swapf   Tmp_STATUS,W            ; restore STATUS
    movwf   STATUS
    swapf   Tmp_WREG,F              ; restore W without clobbering STATUS reg
    swapf   Tmp_WREG,W
    return
		
						
;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Multiple
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
    call    Save_Regs               ; save W, STATUS, & PCLATH
    BANKSEL	0
    bcf     PIR1,CCP1IF             ; Clear Timer 1 request flag
    movlw   0x04		
    xorwf   PORTC,F                 ; toggle CCP1 pin connected to buzzer
    decf    CCPR1L,F                ; decrement value that trigger interrupt

Finish_Int                          ; restore W,STATUS & PCLATH
    call    Restore_Regs
    retfie 
    
    
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;
