;*******************************************************************************;
;   Microchip Technology Inc   2006
;	
;   ASP 201    LAB 6
;   ADC Modifying PWM
;   v2.0 revised 10/3/2006 by Stu Chandler, Sr.TTE
;
;   This lab first sets up the CCP as a PWM and emits a tone identical
;   to that used in lab 4.  The lab then uses the ADC to read the value
;   of the potentiometer on the APP001 and uses this value to modify
;   the volume of the tone.  The modification is accomplished by writing
;   the 8 highest bits of the ADC value into PR2 to set the Period then divided by 2 
;   for the Duty Cycle with CCPR1L.
;   The ADC result are also output to the LEDs.
;
;   The read of the ADC is done in the main control loop using a polling 
;   method and does not use an interrupt.
;
;   The PWM code and the writting to the LEDs are given.  To complete
;   the lab 6 sections of code must be done
;
;   1) Configure the ADC to return a Left Justified value in ADDRESH from
;      channel 0 ( The one coneected to the Pot. on the APP001 )
;
;   2) The main control program that initiates the conversion then waits
;      for the conversion to be completed
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;      
;*******************************************************************************;

; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;=================================== DATA MEMORY

;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------

VARIABLES   UDATA   0x030           ; start placing variables at this location

Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register


;=================================== PROGRAM MEMORY			
			
;-----------------------------------
; VECTORS
;-----------------------------------

RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program

;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;	
	banksel	TRISC
    bcf     TRISC,2                 ; set CCP1 PORTC<RP2> pin as output pin
    clrf    TRISD                   ; configure PORTD as output
;
	Banksel	0
    movlw   0x0C                    ; CCP1 in PWM mode, CCP LSBs to 0
    movwf   CCP1CON
			 
;-----------------------------------
;   configure Timer 2   Pre and post scale of 1:16
	movlw	b'01111111'
    movwf     T2CON              

;-----------------------------------
    
; ### ensure default Channel is set to channel 0
; ### set Tad = 8 Tosc
; ### turn on ADC 
; ### point to BANK1
; ### left justify and set configuration
; ### ADCON1				

    bcf     INTCON,GIE              ; disable global interrupts
    bcf     STATUS,RP0              ; return to BANK0	
	

Loop
; ### start ADC conversion
; ### Is the conversion done?
; ###  no: loop until done

    movf    ADRESH,W                ; move ADC result into W
    movwf   PORTD                   ; ADC result output to LEDs   
	bcf		STATUS,C
	rrf		ADRESH,W
	movwf	CCPR1L 
;            
	movf	ADRESH,W
	banksel	PR2	
    movwf   PR2
	banksel	0    
    goto    Loop 
		
		

;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;
