;*******************************************************************************;
;   Microchip Technology Inc   2007
;	
;   ASP 201    LAB 7
;   USART for the Asynchronous mode
;   v2.0 revised 24 Apr. 2007 by Richard Yang, Pr.CAE
;
;   This lab first config the USART with Asynchornous mode and emits the communication
;   protocol with 9600, None Parity, 8-bit data and 1 stop format.
;   Receive a RS-232 data byte form the Hyper-Terminal in the interupt routine
;   send back the reveived data in the Main program to Hyper-Terminal
;
;   The Received data are also output to the LEDs.
;
;   The EUSART code and the writting to the LEDs are given.  To complete
;   the lab 7 sections of code must be done
;
;   1) Configure the EUSART for the Asynchornous Mode and the RS-232 protocol
;
;   2) Enable the interrupt for the serial data receiving, the routine also puts the
;      received data on the LEDs for the verification
;
;   3) Send back the received data in the main program
; 
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;      
;*******************************************************************************;

; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;=================================== DATA MEMORY

;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------

VARIABLES   UDATA   0x030           ; start placing variables at this location

Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register
;
Rec_Flag	res	1					; Flags.0 = 1 if new data received 
Rec_Data	res	1
S_Hold_Delay 	res 1
Table_Index		res	1
Table_Buffer	res	1

;=================================== PROGRAM MEMORY			
			
;-----------------------------------
; VECTORS
;-----------------------------------

RESET_V     CODE    0x0000          ; reset vector
	clrf	PCLATH
    goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          ; interrupt vector
    goto    Int_Service_Routine     ; jump to interrupt service routine
    
;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS	 			; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;	
	call  Init_USART				; Initialize the UART

	banksel TRISD					; PORTD is output for LEDs
	clrf 	TRISD
	banksel	PORTD
	clrf	PORTD

	call	Send_String1			; Send RS-232 to Terminal for test the 
									; communication protocol setting
	clrf	Rec_Flag				; clear received flag
	clrf	Rec_Data				; clear receive buffer
;
Loop
	btfss	Rec_Flag,0				; Received a RS-232 data from Hyper-Terminal ?
	goto 	Loop		
	bcf		Rec_Flag,0
	call	Send_String2			; send back the reciived data via RS-232
	movf	Rec_Data,W
	call	Send_1_Byte
	movlw	0x0a
	call	Send_1_Byte
	movlw	0x0d
	call	Send_1_Byte
	goto	Loop

;-----------------------------------------

Send_String1						;lookup table for display string on terminal
			clrf	Table_Index
Get_Next	movf	Table_Index,W
			call	String_Table_1
			movwf	Table_Buffer
			sublw	0x00
			btfsc	STATUS,Z
			goto	$+6
			movf	Table_Buffer,W
			call	Send_1_Byte
			incf	Table_Index,F
			goto	Get_Next
			goto	S_Hold_Time

;------------------------------------------------

String_Table_1
	addwf	PCL,F
	dt		0x1b,'[','2','J'					; Clear VT-100 Terminal Screen
	dt		0x1b,'[','0','0',';','0','0','H'	; Set course at 0,0 position
	dt      "Lab7-EUSART, Test :9600,N,8,1"
	dt		0x0a,0x0d,0x00

Send_String2						;lookup table for display string on terminal
			clrf	Table_Index
Get_Next_1	movf	Table_Index,W
			call	String_Table_2
			movwf	Table_Buffer
			sublw	0x00
			btfsc	STATUS,Z
			goto	$+6
			movf	Table_Buffer,W
			call	Send_1_Byte
			incf	Table_Index,F
			goto	Get_Next_1
			goto	S_Hold_Time

String_Table_2
	addwf	PCL,F
	dt		0x0a,0x0d,"Send RxD Data = "
	dt		0x00

;***********************************************************************
;****		Initial USART as 9600,N,8,1 
;***********************************************************************
Init_USART
	banksel BAUDCTL					; Bank 3
	movlw	b'00000000'				; disable Auto-Baud Detect, TxD is RC6, BRG = 8-bit
	movwf	BAUDCTL					

	banksel	TXSTA					; Bank 1
; ### 8-bit data mode , ASYNC 
; ### High Speed mode, Enable TxD
;			
; ### Set baud rate at 9600 with High Speed mode
; ### System Clock are 4MHz using internal RC 

; ### Disable TxD interrupt
; ### Enable RxD interrupt

; ### set input for RC7, RxD receiving pin
; ### set output for RC6, TxD pin

	banksel 0
; ### Enable Serial Port, 8-bit receive
; ### Continuous Receive, Disable Address Detection
;
	bcf		PIR1,TXIF				; Clear TxD interrupt flag;
	bcf		PIR1,RCIF				; Clear RxD interrupt flag				
 			
;		
	bsf		INTCON,PEIE
 	bsf 	INTCON,GIE
			
	return
			


Send_1_Byte:
	banksel	TXREG	
	movwf	TXREG
	btfsc	TXSTA,TRMT
	goto	$-1
S_Hold_Time:
;***********************************************************************
;****		Sample Hold (Charge) time delay routine (50uS) 
;***********************************************************************
   	movlw	.255
   	movwf	S_Hold_Delay
	nop
   	decfsz	S_Hold_Delay,F
   	goto	$-2
   	return


;-----------------------------------
; Subroutine:   Save_Regs
; Parameters:   WREG, STATUS, PCLATH
; Returns:      Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Synopsys:     Saves WREG, STATUS & PCLATH upon entering interrupt
;-----------------------------------

Save_Regs                           ; routine to save W, STATUS, & PCLATH
    movwf   Tmp_WREG                ; save WREG in temp register			
    swapf	STATUS,W                ; swapf does not clobber status bits
    movwf	Tmp_STATUS              ; save STATUS in temp register
    movf	PCLATH,W                ; save PCLATH
    movwf	Tmp_PCLATH
    return
		

;-----------------------------------
; Subroutine:   Restore_Regs
; Parameters:   Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Returns:      WREG, STATUS, PCLATH
; Synopsys:     Restores WREG, STATUS & PCLATH after interrupt
;-----------------------------------

Restore_Regs						; rountine to restore W, STATUS, & PCLATH
    movf    PCLATH,W                ; restore PCLATH
    movwf   PCLATH
    swapf   Tmp_STATUS,W            ; restore STATUS
    movwf   STATUS
    swapf   Tmp_WREG,F              ; restore W without clobbering STATUS reg
    swapf   Tmp_WREG,W
    return
		

;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Multiple
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
    call    Save_Regs               ; save W, STATUS, & PCLATH
    
    BANKSEL	0				        ; select bank 0
	movf	RCREG,W					; Clear the RCIF and move received data to W
	movwf	Rec_Data
	movwf	PORTD
	bsf		Rec_Flag,0				; Set receiving flag for main program

Finish_Int                          ; restore W,STATUS & PCLATH
    call    Restore_Regs
    retfie     
		
		
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;


