;*******************************************************************************;
;   Microchip Technology Inc   2006
;
;   ASP 201     LAB 8
;   I2C lab
;   v2.0 revised 9/29/2006 by Stu Chandler, Sr.TTE
;
;   This lab configures the MSSP as an I2C controller, then reads the value from 
;   the TC74 temperature sensor over the I2C bus on the APP001.  After 
;   reading the temperature sensor the value is displayed on the LEDs.
;
;   To complete this lab three sections of code must be completed:
;
;       1) Set the slew rate and buad rate for 100Khz operation
;       2) Configure the MSSP to operate as an I2C master 7-bit mode and enable
;          SDA and SCL
;       3) Add a section of the main loop to issue the START condition
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;		      
;*******************************************************************************;
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions

;
;***********      The subroutines for I2C      *****************
;
;	Init_I2C_Master		; Initial I2C Module for Master Mode , 7-bit address
;	StartI2C		; Send a START Condition !!
;	StopI2C			; Send s STOP Condition
;	RstartI2C		; Send a Repeat Start conditional to I2C
;	Non_Ack			; Send a Non-acknowledge signal to I2C
;	An_Ack			; Send a acknowledge signal to I2C
;	Sebd_Byte		; Send a byte to I2C bus
;	RecI2C	 		; Enable Read a byte form slave device
;	I2C_Done		: Wait the I2C completed the currect process
;
;***************************************************************************************** 

;***************************************
;       wqܼƦbRAM}
;
 			UDATA	0x30
I2C_Ctrl	res	1 
I2C_Addr	res	1
I2C_Data	res	1
D_Count		res	1
;
;***************************************
;       

w_temp		EQU	0x72
status_temp	EQU	0x73
pclath_temp	EQU	0x74 
;
;***************************************
;       wq I2C }θ}
;
#define 	TC74_Addr	b'10011110'	; Device adress of Slave Point
#define		TC74_RTC	0x00		; Read Temperature command
#define		TC74_RWCR	0x01		; Read/Write Configuration command
#define		EE_Read		.1
#define		EE_Write	.0
;
#define 	SCL	PORTC,3		; I2C SCL pin
#define		SDA	PORTC,4		; I2C SDA pin

;

;********************************************
; Locates startup code @ the reset vector
;********************************************
Reset_Addr
RESET_V		code	0x00
			nop
			goto	Main_Init
;
INT_V		code	0x04
			retfie 

	 	
 
;----------------------------------------------------------------------
			code	0x0030
Main_Init   
			banksel	TRISD
			clrf	TRISD 
   			call    Init_I2C_Master		; Init the MSSP for I2C Master 
;
Read_TC74	
			banksel	I2C_Ctrl
			movlw	TC74_Addr			; Load TC74 slave address b'10011110'
			movwf	I2C_Ctrl
			movlw	TC74_RTC			; Read Temperature command
			movwf	I2C_Addr
			call	EE_Random_Read
			movf	I2C_Data,W			; Write Temperature value to LEDs
			movwf	PORTD
;
			call	Delay_mS			; Delay 1mS
			call	Delay_mS
			goto 	Read_TC74	
;

;
;******************************************************
;*           Random Read a Byte from I2C EEPROM          
;*
;* Input:
;*  		 - I2C_Ctrl : Control Byte of EEPROM 		 
;*   		 - I2C_Addr : Location of EEPROM		 
;* Output:
;*   		 - I2C_Data : Read Data from EEPROM			 
;******************************************************
;
; Send Command for RANDOM READ : 
; " Start+ 0xA0 + EE_Address + ReStart + 0xA1 + Read_Data + NAck + Stop "
;

EE_Random_Read
 			call	StartI2C		; Set SSPCON2.SEN
;	
			bcf		I2C_Ctrl,0		; set for write Command
			movf	I2C_Ctrl,W		; Send Slave Address to I2C Bus
			call	Send_Byte 
;
			movf	I2C_Addr,W		; Send out the Rendom address of EEPROM	 
			call	Send_Byte 
;
			call	RstartI2C		; Send a Repeat Start to I2C
;
			bsf		I2C_Ctrl,0		; set for Read Command
			movf	I2C_Ctrl,W		; Send Slave Address to I2C Bus
			call	Send_Byte 
;
			call	RecI2C		; Enable I2C Receive 
;
			BANKSEL	SSPBUF
			movf	SSPBUF,W	; Save to I2C_Data First !!	
			movwf	I2C_Data

			call	Non_Ack		; Initial NACK Response !!

			call	StopI2C		; Initial STOP Condition 
			return
;

;******************************************************
;*              EEPROM Acknowledge Polling
;*          	 
;*   	 --  The routine will polling the ACK		 
;*           response from EEPROM   				 
;*  	 --  ACK=0 return 							 
;*		 --  ACK=1 send Restart & loop check	
;*	 
;******************************************************
;
EE_Ack_Check
 			call	StartI2C		; Set SSPCON2.SEN
 			
			bcf		I2C_Ctrl,0		; Clear for Write Command	
			movf	I2C_Ctrl,W		; Send Slave Address to I2C Bus
			call	Send_Byte 
_Ack_Polling	
			BANKSEL	SSPCON2
			btfss	SSPCON2,ACKSTAT	; Check ACKSTAT bit , 0 = ACK , 1 = NACK
			goto	ACK_Return		; Ack = 0 ; EEPROM is Ready
_Ack_Hi								; Ack = 1 ; EEPROM is Busy for Write
			call	Delay_mS		; Delay 500uS for next Ack polling
			call	RstartI2C		; Send a Repeat Start to I2C
			bcf		I2C_Ctrl,0		; Clear for Write Command
			movf	I2C_Ctrl,W		; Send Slave Address to I2C Bus
			call	Send_Byte 
			goto	_Ack_Polling
	
ACK_Return 
			call	StopI2C			; Initial STOP Condition 
			return

;
;*******************************************************
;*               Write a Byte to EEPROM                
;*
;* Input:
;*  		 - I2C_Ctrl : Control Byte of EEPROM 		 
;*   		 - I2C_Addr : Location of EEPROM		 
;*   		 - I2C_Data : Data to EEPROM
;*			
;*******************************************************
;
; Send Command for BYTE WRITE : 
; " Start+ 0xA0 + EE_Address + Write_Data + Stop "
;
EE_Byte_Write
 			call	StartI2C		; Set SSPCON2.SEN
;	
			bcf		I2C_Ctrl,0		; Clear for Write Command
			movf	I2C_Ctrl,W		; Send Slave Address to I2C Bus
			call	Send_Byte 
;
			movf	I2C_Addr,W		; Send out the Command	 
			call	Send_Byte 
;
			movf	I2C_Data,W		; Send out the Data	
			call	Send_Byte 
;
 			call	StopI2C			; Initial STOP Condition 
			return
;
;**********************************************************************
;
;   As following subroutines perform commonly used I2C functions.
;      You can use these subroutines for your I2C access
;
;**********************************************************************
;
;*** Initial I2Cmodule fot 7-bit I2C Master Mode with 100Kcps @4MHz
;
Init_I2C_Master				 
			BANKSEL	TRISC			; Initial PortC,bit 3 & 4 as Input
 			bsf		SCL				; RC3 = SCL  ,  RC4 = SDA
			nop
			bsf		SDA		 
;
			BANKSEL	SSPCON
			movlw	b'00101000'		; ### I2C Master Mode, Clock Rate: FOSC/(4*SSPADD+1)
			movwf	SSPCON			; ###
;
        	banksel SSPADD
	        movlw   .9             	; ### This gives 100KHz I2C clock @ 4MHz
			movwf   SSPADD			; ### (4MHz/4) / (9+1)= 100KHz
;
     		movlw   b'10000000'     ; ### Disable slew rate control,
        	movwf   SSPSTAT			; ### and clear status bits
;
 	        movlw   b'00000000'     ; Set SCL,SDA into Ready status
        	movwf   SSPCON2          
;     
			return
;
;******   Send START condition to bus   *******
;
StartI2C				; Initiate the I2C START condition.
   	   		banksel SSPCON2
     	   	bsf     SSPCON2,SEN
     	  	goto	I2C_Done
;
;******   Send STOP condition to bus   *******
;
StopI2C				 
      		banksel SSPCON2
        	bsf     SSPCON2,PEN
        	goto	I2C_Done
;
;******   Send RESTART condition to bus   *******
;      
RstartI2C			 
        	banksel SSPCON2
        	bsf     SSPCON2,RSEN
        	goto	I2C_Done
;
;******   Send a Non-Acknowledge status to bus (ACK=1)    *******
;        
Non_Ack
        	banksel SSPCON2
        	bsf     SSPCON2,ACKDT    ; Set the ACK bit
        	bsf     SSPCON2,ACKEN    ; Initiate the NACK sequence.
        	goto	I2C_Done
;
;******   Send an Acknowledge status to bus (ACK=0)     *******
;         
An_Ack
       	 	banksel SSPCON2
        	bcf     SSPCON2,ACKDT    ; Clear the ACK bit
        	bsf     SSPCON2,ACKEN    ; Initiate the NACK sequence.
        	goto	I2C_Done
;
;******   Send data to I2C bus from Wreg.     *******
; 
Send_Byte
			banksel	SSPBUF
			movwf	SSPBUF
			goto	I2C_Done
;
;******   Enable I2C Receive for Master Mode     *******
; 
RecI2C
      		banksel SSPCON2          
        	bsf     SSPCON2,RCEN    ; Set the receive enable bit.
			goto	I2C_Done
;
;******  Check the I2C stage is completed   ********
; 
I2C_Done
	       	banksel PIR1
 			btfss   PIR1,SSPIF	; Poll for SSPIF
        	goto    $-1
        	bcf     PIR1,SSPIF
        	return
;
Delay_mS
			banksel	D_Count
			movlw	.124			; Load .5mS Dealy Value
 			movwf	D_Count			 
_D_mS		nop
			decfsz	D_Count,F		 
			goto	_D_mS		
			return					     
;
; *********** End Of Program !!!!!
;
			end
