;*******************************************************************************;
;   Microchip Technology Inc   2006
;
;   ASP 201     LAB 8
;   I2C lab
;   v2.0 revised 9/29/2006 by Stu Chandler, Sr.TTE
;
;   This lab configures the MSSP as an I2C controller, then reads the value from 
;   the TC74 temperature sensor over the I2C bus on the APP001.  After 
;   reading the temperature sensor the value is displayed on the LEDs.
;
;   To complete this lab three sections of code must be completed:
;
;       1) Set the slew rate and buad rate for 100Khz operation
;       2) Configure the MSSP to operate as an I2C master 7-bit mode and enable
;          SDA and SCL
;       3) Add a section of the main loop to issue the START condition
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;		      
;*******************************************************************************;
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;=================================== DATA MEMORY	

;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location
Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register
cmd_byte    RES 1

;=================================== PROGRAM MEMORY
			
;-----------------------------------
; VECTORS
;-----------------------------------

RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program


INT_V       CODE    0x0004          ; interrupt vector

;   No interupts expected, put in retfie in case of unexpected interrupt
    
    retfie


;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------

            CODE    0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;		
    BANKSEL TRISC                   ; initialize MSSP module
    bsf     TRISC,3
    bsf     TRISC,4
    clrf    TRISD                   ; make PORTD an output


;-----------------------------------
; set the slew rate and baud rate for 100 KHZ operation

    BANKSEL SSPSTAT                 ; ### point to correct BANK
    bsf     SSPSTAT,SMP             ; ### Set for Standard speed slew rate
    movlw   .5
    movwf   SSPADD                  ; set the address of the slave device


;-----------------------------------
; Configure as I2C master with Fosc/4 Clock source

    BANKSEL SSPCON
    bsf     SSPCON,SSPM3            ; ### set to I2C master mode with Fosc/4 clock source
    bsf     SSPCON,SSPEN            ; ### Enable SDA and SCL pins to operate in I2C mode

    BANKSEL SSPCON2                 ; ensure SSPCON2 is cleared
    clrf    SSPCON2


;----------------------------------- 
; 	Put '0000001' into variable cmd_byte.
;   This is the read commnd that will be sent to the TC74

    BANKSEL cmd_byte
    movlw   0x00                    ; Read Temperature command byte
    movwf   cmd_byte


;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------    

Loop
	
;-----------------------------------

    BANKSEL SSPCON2	                ; Initiate a START condition		
    bsf     SSPCON2,SEN             ; ### set the SEN bit
    btfsc   SSPCON2,SEN             ; ### is it finished?
    goto	$-1	                    ;   no: test again

;-----------------------------------

    movlw   B'10011110'             ; send address of TC74 w/0 in the LSB
    BANKSEL SSPBUF	                ; send TC74 ADDRESS (write)
    movwf   SSPBUF
	BANKSEL	SSPSTAT
    btfsc   SSPSTAT,R_W             ; wait for write to complete
    goto    $-1

;-----------------------------------

 					                ; Wait for ACK from TC74
    btfsc   SSPCON2,ACKSTAT         ; ack?
    goto    $-1

;-----------------------------------

	BANKSEL cmd_byte                ; send the READ command
    movf    cmd_byte,W              ;   send COMMAND byte (config)
    
    BANKSEL SSPBUF
    movwf   SSPBUF
;    movlw   0x00
    
;   BANKSEL SSPCON2
;    andwf	SSPCON2,w
;    sublw   0x00
;    btfss   STATUS,Z
;    goto    $-4
	BANKSEL	SSPSTAT
    btfsc   SSPSTAT,R_W
    goto    $-1

;-----------------------------------

    BANKSEL SSPCON2                 ; get ACK from TC74
    btfsc   SSPCON2,ACKSTAT         ;  ack?
    goto    $-1

    bsf     SSPCON2,RSEN            ; resend START condition   
    btfsc   SSPCON2,RSEN            ; send repeated start
    goto    $-1

    movlw	B'10011111'             ; send TC74 ADDRESS
    BANKSEL SSPBUF                  ;  with READ bit set
    movwf	SSPBUF
    movlw	0x00
    
    BANKSEL SSPSTAT
;    andwf	SSPCON2,w
;    sublw	0x00
;    btfss	STATUS,Z
;    goto	$-4
    btfsc	SSPSTAT,R_W
    goto	$-1

;-----------------------------------
						
    BANKSEL SSPCON2                 ; get ACK from TC74
    btfsc   SSPCON2,ACKSTAT         ; ack?
    goto    $-1

    bsf     SSPCON2,RCEN            ; receive temp data
    btfsc   SSPCON2,RCEN            ; enable receive mode
    goto    $-1
    BANKSEL SSPBUF                  ; retrieve temp reg
    movf    SSPBUF,w                ; put measured temperature into WREG

;-----------------------------------

NACK	
    BANKSEL SSPCON2                 ; send NOT-ACK condition
    bsf     SSPCON2,ACKDT
    bsf     SSPCON2,ACKEN
    btfsc   SSPCON2,ACKEN
    goto    $-1

STOP			
    bsf     SSPCON2,PEN             ; send stop condition
    btfsc   SSPCON2,PEN
    goto    $-1
    
;-----------------------------------

    BANKSEL PORTD                   ; send LSB of temp data to PORTB
    movwf   PORTD
    nop
    goto    Loop

;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;
