;*******************************************************************************;
;   Microchip Technology Inc   2006
;
;   ASP 201     LAB 9
;   Multiple Interrupts lab
;   v2.0 revised 9/29/2006 by Stu Chandler, Sr.TTE
;
;   This lab demonstrates a system with multiple interrupts.  The lab
;   combines the LAB5 Output Compare with the LAB1 INT0 concepts 
;   On a CCP interrupt, the vector looks at a variable (push_flag). If 
;   push_flag is set the value of CCPR1L is decremented.  If push_flag is
;   not set then no change is made in CCPR1L.  If push_flag is set the 
;   buzzer will chirp, if cleared the tone remains the same.  push_flag
;   is toggled by the interrupt vector handling INT 0
;
;   The two interrupt vectors  INT_ISR and CCP_ISR have been provide. To
;   complete the lab two sections of code need to be completed
;
;   1)  check the pending interrupt flags to determine which interrupt
;       occured then call the appropriate ISR
;
;   2) enable all the interrupts needed ( Global, Peripheral, CCP1, INT0)
;
;   The locations of the code to be added by the student are marked by comments 
;   containing ### Your Code Here ###
;		      
;*******************************************************************************;

;=================================== DATA MEMORY
; Request : MPLAB IDE v7.52 or higher
;===================================  

			list		p=16f887		; list directive to define processor
			#include	<p16f887.inc>	; processor specific variable definitions
	
;-----------------------------------
; USER DEFINED VARIABLES
;-----------------------------------
		
VARIABLES   UDATA   0x030           ; start placing variables at this location

Tmp_WREG    RES 1                   ; temporary storage for W register
Tmp_STATUS  RES 1                   ; temporary storage for STATUS register
Tmp_PCLATH	RES 1                   ; temporary storage for PCLATH register

push_flag   RES 1                   ; Flag to indicate whether or not S3 was pressed
db_ctr1     RES 1                   ;
db_ctr2     RES 1                   ; debounce counters			


;=================================== PROGRAM MEMORY			
			
;-----------------------------------
; VECTORS
;-----------------------------------

RESET_V     CODE    0x0000          ; reset vector
    goto    Main                    ; jump to start of program

INT_V       CODE    0x0004          ; interrupt vector
    goto    Int_Service_Routine     ; jump to interrupt service routine
    

;-----------------------------------
; INITIALIZATION
;-----------------------------------

            CODE	0x0040          ; force code to start on page 0            
Main
;===================================  System Clock Setting
;   Set the system clock at 4MHz using Hi-Frequency Internal RC
;   Set the OSCCON<IRCF2:IRCF0> for 4MHz selection
;   Internal RC Oscillator (INTOSC) is used for the system clock
	
	banksel OSCCON
	movlw	b'01100001'				; <ICRF2:ICRF0> = 110, <SCS> bit = 1 for INTOSC
	movwf	OSCCON
	nop
	btfss	OSCCON,HTS				; Check the HFINTOSC ststus?
	goto	$-2						; HFINTRC is not stable. Check again.
;	
;-----------------------------------
; Set CCP1COM to Output Compare mode with Special Event Trigger
; to clear the TMR1 register pair on a match

	banksel	CCP1CON
    movlw   0x0B        
    movwf   CCP1CON                 ; set value in CCP1CON

;-----------------------------------

    movlw   0x30                    ; config TMR1 for Fosc/4, 8:1 Pre-scaler
    movwf   T1CON		

    clrf    CCPR1L                  ; init CCP1 to 0x0100
    clrf    CCPR1H
    incf    CCPR1H,F
    			 
;-----------------------------------
;		Configure CCP1 pin as an ouput and INT0 as an input
;
	banksel	ANSELH					; switch to bank 3
	bcf		ANSELH,ANS12			; disable AD input function for RB0/AN12
;
    banksel	TRISC              ; go to BANK1
    bcf     TRISC,2                 ; set CCP2 to an output			
    bsf     TRISB,0                 ; set SW2 to input

;-----------------------------------

    bsf     PIE1,CCP1IE	            ; ### enable CCP1 interrupt
    bsf     INTCON,INTE             ; ### enable INT0 interrupt
    bsf     INTCON,PEIE             ; ### enable peripheral interrupts
    bsf     INTCON,GIE              ; ### enable global interrups	
    bcf     STATUS,RP0				; return to BANK0

    bsf     T1CON,TMR1ON            ; turn on TMR1


;-----------------------------------
; MAIN PROGRAM LOOP
;-----------------------------------	

Loop    nop
        goto    Loop 

;-----------------------------------
;   CCP  Interrupt Service Routine

CCP_ISR							
    BANKSEL	0                       ; ensure Page 0 selected
    bcf		PIR1,CCP1IF             ; Clear Timer 1 request flag
    movlw	0x04		
    xorwf	PORTC,F                 ; toggle CCP1 pin connected to buzzer
    movlw	0xFE                    ; load -2 into WREG
    btfsc	push_flag,0             ; did INT0 set the push_flag?
    movlw	0x00                    ; yes: clear WREG
    addwf	CCPR1L,F                ; increment value that triggers interrupt
    call 	Restore_Regs
    retfie

;-----------------------------------
;   INT0 (S3) Interrupt Service Routine

INT0_ISR
    call    Debounce                ; Wait until SW2 quits "bouncing"
    movlw   0x01			
    xorwf   push_flag,F             ; toggle bit 0 of push_flag
    bcf     INTCON,INTF             ; clear Interrupt flag
    call    Restore_Regs
    retfie					


;-----------------------------------
; Subroutine:   Save_Regs
; Parameters:   WREG, STATUS, PCLATH
; Returns:      Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Synopsys:     Saves WREG, STATUS & PCLATH upon entering interrupt
;-----------------------------------

Save_Regs                           ; routine to save W, STATUS, & PCLATH
    movwf   Tmp_WREG                ; save WREG in temp register			
    swapf	STATUS,W                ; swapf does not clobber status bits
    movwf	Tmp_STATUS              ; save STATUS in temp register
    movf	PCLATH,W                ; save PCLATH
    movwf	Tmp_PCLATH
    return


;-----------------------------------
; Subroutine:   Restore_Regs
; Parameters:   Tmp_WREG, Tmp_STATUS, Tmp_PCLATH
; Returns:      WREG, STATUS, PCLATH
; Synopsys:     Restores WREG, STATUS & PCLATH after interrupt
;-----------------------------------		

Restore_Regs						; rountine to restore W, STATUS, & PCLATH
    movf    PCLATH,W                ; restore PCLATH
    movwf   PCLATH
    swapf   Tmp_STATUS,W            ; restore STATUS
    movwf   STATUS
    swapf   Tmp_WREG,F              ; restore W without clobbering STATUS reg
    swapf   Tmp_WREG,W
    return			


;-----------------------------------
; Subroutine:   Debounce
; Parameters:   none
; Returns:      none
; Synopsys:     loop for debouncing S3
;-----------------------------------

Debounce                            ; allows time for SW2 to debounce
    incfsz      db_ctr1,F
    goto        $-1
    incfsz      db_ctr2,F
    goto        Debounce
    return 	


;-----------------------------------
; Subroutine:   Int_Service_Routine
; Parameters:   None
; Returns:      Multiple
; Synopsys:     System ISR
;-----------------------------------

Int_Service_Routine	
    call    Save_Regs               ; save W, STATUS, & PCLATH
		
    btfsc   INTCON,INTF             ; ### test for INT0 interrupt request
    goto    INT0_ISR
    btfsc   PIR1,CCP1IF             ; ### test for CCP Interrupt request
    goto    CCP_ISR			

;-----------------------------------
;   This section of code should will not be executed unless an interrupt
;   other than INT0 or CCP1 occurred

Finish_Int                          ; restore W,STATUS & PCLATH
    call    Restore_Regs
    retfie     
		
		
;-----------------------------------

	END                             ; directive telling MPASM that code is done	
	
	
;*******************************************************************************;
;                 CODE OWNERSHIP AND DISCLAIMER OF LIABILITY                    ;
;                                                                               ;
; Microchip Technology Incorporated ("Microchip") retains all ownership and     ;
; intellectual property rights in the code accompanying this message and in all ;
; derivatives hereto.  You may use this code, and any derivatives created by    ;
; any person or entity by or on your behalf, exclusively with Microchips       ;
; proprietary products.  Your acceptance and/or use of this code constitutes    ;
; agreement to the terms and conditions of this notice.                         ;
;                                                                               ;
; CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY MICROCHIP "AS IS". NO           ;
; WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED ;
; TO, IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A ;
; PARTICULAR PURPOSE APPLY TO THIS CODE, ITS INTERACTION WITH MICROCHIPS       ;
; PRODUCTS,COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.      ;
;                                                                               ;
; YOU ACKNOWLEDGE AND AGREE THAT, IN NO EVENT, SHALL MICROCHIP BE LIABLE,       ;
; WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF        ;
; STATUTORY DUTY), STRICT LIABILITY, INDEMNITY,CONTRIBUTION, OR OTHERWISE, FOR  ;
; ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, ;
; DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,       ;
; HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY OR    ;
; THE DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW,         ;
; MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED; TO THIS CODE,   ;
; SHALL NOT EXCEED THE PRICE YOU PAID DIRECTLY TO MICROCHIP SPECIFICALLY TO     ;
; HAVE THIS CODE DEVELOPED.                                                     ;
;                                                                               ;
; You agree that you are solely responsible for testing the code and            ;
; determining its suitability.  Microchip has no obligation to modify, test,    ;
; certify, or support the code.                                                 ;
;*******************************************************************************;

